/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/* ADT of chained hash table operations.
 */
#ifndef _HASH_H
#define _HASH_H

#include <stddef.h>
#include "ulbool.h"
#include "uldef.h"
_BEGIN_DECLS

typedef struct hash_node_s {
	const void *data;		/* const pointer to hash data source. */
	const char *key;		/* const pointer to hash key source. */
	struct hash_node_s *next;	/* pointer to next node. */
} hash_node_t; 				/* ADT of chained hash node. */

typedef struct {
	hash_node_t *head; 		/* pointer to head node. */
} hash_buck_t; 				/* ADT of hash bucket. */

/* string hash function. */
typedef unsigned (*hash_func)(const char *src);

typedef struct {
	hash_buck_t* memstart;		/* initial bucket in array. */
	size_t maxbucks; 		/* primary number strongly preferred. */
	hash_func hashfunc; 		/* string hash function. */
} hash_t; 				/* ADT of chained hash talbe. */

_END_DECLS
#endif
