/* Copyright (C) 2008, Cyril Hu (cyrilhu@gmail.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#if defined(LIB_DEBUG) || defined(LIB_ARRAY_DEBUG)
#include <stdio.h>
#include <string.h>
#include "ullib.h"
#endif
#include <stddef.h>
#include "ulbool.h"

/* To see whether array is keyword ordered or not,
 * return true if ordered, false if not. */
bool array_issorted(const void *src, size_t n, size_t sz, int (*cmp)(const void *, const void *))
{
	unsigned char *p, *q;
	int t,flaga=0,flagd=0;
	size_t i;
	for(i=0; i < n-1; ++i) {
		p = (unsigned char *)src + i*sz, q = p + sz;
		t = (*cmp)(p, q);
		if(t < 0) {
			flaga=1;
		} else if(t > 0) {
			flagd=1;
		}
	}
	return flaga && flagd ? false : true;
}

/* Find array's min value or max value element,
 * return a pointer to this element. */
void *array_getmaxormin(const void *src, size_t n, size_t sz, int (*cmp)(const void *, const void *))
{
	const unsigned char *p = src, *q;
	size_t i;
	for(i=1; i < n; ++i) {
		q = (const unsigned char *)src + i * sz;
		if( (*cmp)(p, q) > 0 )
			p = q;
	}
	return (void *)p;
}

#ifdef LIB_ARRAY_DEBUG
int main(void)
{
	int buf[256], i=0;

	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpa));
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpd));
	if(array_issorted(buf, 256, sizeof(int),icmpa))
		puts("array_issorted !");
	else
		puts("array is not sorted !");
	if(array_issorted(buf, 256, sizeof(int),icmpd))
		puts("array_issorted !");
	else
		puts("array is not sorted !");

	for(; i<256; ++i) {
		/* printf("buf[%d]=%d\n", i,i);
		 */
		buf[i] = i;
	}
	
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpa));
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpd));
	if(array_issorted(buf, 256, sizeof(int),icmpa))
		puts("array_issorted !");
	else
		puts("array is not sorted !");
	if(array_issorted(buf, 256, sizeof(int),icmpd))
		puts("array_issorted !");
	else
		puts("array is not sorted !");

	buf[234]=430;
	
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpa));
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpd));
	if(array_issorted(buf, 256, sizeof(int),icmpa))
		puts("array_issorted !");
	else
		puts("array is not sorted !");
	if(array_issorted(buf, 256, sizeof(int),icmpd))
		puts("array_issorted !");
	else
		puts("array is not sorted !");
	
	for(i=0; i<256; ++i) {
		/* printf("buf[%d]=%d\n", i,i);
		 */
		buf[i] = 1000-i;
	}
	
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpa));
	printf("min or max=%d\n", *(int *)array_getmaxormin(buf, 256, sizeof(int), icmpd));
	if(array_issorted(buf, 256, sizeof(int),icmpa))
		puts("array_issorted !");
	else
		puts("array is not sorted !");
	if(array_issorted(buf, 256, sizeof(int),icmpd))
		puts("array_issorted !");
	else
		puts("array is not sorted !");

	return 0;
}
#endif

